/*
		Copyright 2000-2002 Frdric Brown
*/

/*
		This file is part of CLib.

    CLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with CLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
	Converts an ANSI string to the TIOS string format
*/
char* stringToVar(char* string)
{
	int length = strlen(string);//The length of the string to convert
	char *var = malloc(sizeof(char) * (length + 2));//Allocate length bytes to var, the pointer to the TIOS formated string
	int i;//A counter that will be used by the for
	
	var[0] = '\0';//TIOS strings begin with null character
	
	//for each character position of var, from 1 up to the length of the string to convert
	for (i=1; i<=length; i++) {
		var[i] = string[i-1];//Assign to the character position i of var the character in position i-1 of the string to convert
	}
	
	var[i] = '\0';//TIOS strings end with null character
	
	return (var + length + 1);//Return a char pointer to the ending null character of the TIOS string
}

/*
 Put a bitmap in the screen from a PIC variable
*/
void show_picvar (char *SymName, int x, int y, int Attr) {
	SYM_ENTRY *sym_entry = SymFindPtr (SymName, 0);//The pointer on the TIOS variable
	//If the TIOS variable doesn't exists
	if (!sym_entry) return;//Quits the function
	//If the TIOS variable isn't a PIC
	if (peek (HToESI (sym_entry->handle)) != PIC_TAG) return;//Quits the function
	BitmapPut (x, y, HeapDeref (sym_entry->handle) + 2, ScrRect, Attr);//Shows the bitmap
}

/*
	Saves a bitmap to a TIOS PIC variable
*/
void savePicvar (char *nom, BITMAP* bmp) {
	short taille = bmp->NumRows*((bmp->NumCols+7)>>3)+4;//The length of the bitmap
	SYM_ENTRY *sym_entry;//A pointer on the TIOS VAT entry
	char *vat,//A pointer to the TIOS variable
			 *var;//Will contain the name of the variable, in TIOS string format

	var = stringToVar(nom);//Convert nom to TIOS string format
	
	sym_entry = DerefSym(SymAdd(var)) ;//Creates a new TIOS entry and assigns it to sym_entry
	
	vat = (unsigned char*)HeapDeref(sym_entry->handle = HeapAlloc(taille+3)) ;//Assigns the adress of the TIOS variable to vat
	//Writes the dimensions of the PIC to the PIC header
	vat[0] =  (taille + 1) / 256;
	vat[1] =  (taille + 1) % 256;
	memcpy(vat+2, bmp, taille) ;//Copy the bitmap data to the TIOS variable
	vat[taille+2] = 0xDF ;//Ends the TIOS variable by the PIC_VAR tag (DF)
	
	free(var - strlen(nom) - 1);//Frees the memory allocated to the TIOS string
}

void DRAWPIX(int x, int y, int arg)
{	
	//Depending on arg
	switch (arg) {
		case 0: {
			DrawPix(x, y, A_NORMAL);//Draws a black pixel
			break;
		}
		case 2: {
			DrawPix(x, y, A_REVERSE);//Draws a white pixel
			break;
		}
		case 3: {
			DrawPix(x, y, A_XOR);//Draws a xor pixel
			break;
		}
	}
}

void DRAWLINE(int x0, int y0, int x1, int y1, int arg)
{
	//Depending on arg
	switch (arg) {
		case 0: {
			DrawLine(x0, y0, x1, y1, A_NORMAL);//Draws a normal line
			break;
		}
		case 1: {
			DrawLine(x0, y0, x1, y1, A_REVERSE);//Draws a reverse line
			break;
		}
		case 2: {
			DrawLine(x0, y0, x1, y1, A_XOR);//Draws a xor line
			break;
		}
		case 3: {
			DrawLine(x0, y0, x1, y1, A_THICK1);//Draws a double thick line
			break;
		}
		case 4: {
			DrawLine(x0, y0, x1, y1, A_SHADE_V);//Draws the line using a vertical shading pattern
			break;
		}
		case 5: {
			DrawLine(x0, y0, x1, y1, A_SHADE_H);//Draws the line using a horizontal shading pattern
			break;
		}
		case 6: {
			DrawLine(x0, y0, x1, y1, A_SHADE_NS);//Draws the line using a negative slope diagonal shading pattern
			break;
		}
		case 7: {
			DrawLine(x0, y0, x1, y1, A_SHADE_PS);//Draws the line using a positive slope diagonal shading pattern
			break;
		}
	}
}

void SHAPE(int nbrParam, int arg, ...)
{
	int x0,//Will contain the first x coordinate
		y0,//Will contain the first y coordinate
		x1,//Will contain the second x coordinate
		y1,//Will contain the second y coordinate
		i;//Will be used by the for as counter
	
	va_list argptr;//A pointer on the function argument stack
	va_start(argptr, nbrParam);//Initialisation of the argument pointer
	x1 = va_arg(argptr, int);//Assign argument 3 to x1 as int
	y1 = va_arg(argptr, int);//Assign argument 4 to y1 as int

	for (i=0; i<nbrParam-1; i++)
	{
		//The last second coordinates become the new first
		x0 = x1;
		y0 = y1;
		x1 = va_arg(argptr, int);//Get the new second x coordinate
		y1 = va_arg(argptr, int);//Get the new second y coordinate
		
		DRAWLINE(x0, y0, x1, y1, arg);//Draws the line
	}
	
	va_end(arg_ptr);
}

void TIMER(int delay)
{	
	OSFreeTimer(6);//Frees the timer 6 (user's timer)
	OSRegisterTimer(6, delay);//Initialize the timer with the delay
	while (!OSTimerExpired(6));//Wait for the timer to expire
}

void DRAWSTR(int x, int y, char* chaine, int arg)
{	
	//Depending on arg
	switch(arg) {
		case 0: {
			DrawStr(x, y, chaine, A_NORMAL);//Draws a normal string
			break;
		}
		case 1: {
			DrawStr(x, y, chaine, A_REVERSE);//Draws a opaque white on black string
			break;
		}
		case 2: {
			DrawStr(x, y, chaine, A_REPLACE);//Draws a opaque black on white string
			break;
		}
		case 3: {
			DrawStr(x, y, chaine, A_XOR);//Draws a xor string
			break;
		}
		case 4: {
			DrawStr(x, y, chaine, A_SHADED);//Draws a shaded string
			break;
		}
	}
}

void BOX(int x0, int y0, int x1, int y1, int arg)
{
	DRAWLINE(x0, y0, x1, y0, arg);//Draws the top side
	DRAWLINE(x0, y1, x1, y1, arg);//Draws the bottom side
	DRAWLINE(x0, (y0+1), x0, (y1-1), arg);//Draws the left side
	DRAWLINE(x1, (y0+1), x1, (y1-1), arg);//Draws the right side
}

void PANEL(int x0, int y0, int x1, int y1, int arg)
{
	int i;//Will be used by the for as counter
	
	//For each pixels, from y0 to y1
	for (i=y0; i<=y1; i++)
	{
		DRAWLINE(x0, i, x1, i, arg);//Draws a horizontal line from (x0,i) to (x1,i)
	}
}

void CLRSCR(void)
{
	clrscr();//Clears the screen :)
}

char* CURSOR(int x0, int y0, int pasX, int pasY)
{
	char 	coorX[50], //A buffer of 50 bytes to stock the x coordinate (+ ',' and the y coordinate when the function will returns)
			coorY[50];//A buffer of 50 bytes to stock the y coordinate
				
	int		key = 0;

	show_picvar($(curimg), x0, y0, A_XOR);//Shows the cursor for the first time

	do
	{
		//When the user presses a key
		if (kbhit ( ))
		{
			key = ngetchx ( );//Assing the key pressed to key
			show_picvar($(curimg), x0, y0, A_XOR);//Hides the cursor
			
			//Depending on the key pressed by the user (can't use a switch here, because KEY_... are macros instead of constants)
			//If the key pressed is the right key
			if (key == KEY_RIGHT)
			{
				x0 = x0 + pasX; //increment the x position
			}
			//If the key pressed is the left key
			else if (key == KEY_LEFT)
			{
				x0 = x0 - pasX;//decrement the x position
			}
			//If the key pressed is the up key
			else if (key == KEY_UP)
			{
				y0 = y0 - pasY;//decrement the y position
			}
			//If the key pressed is the down key
			else if (key == KEY_DOWN)
			{
				y0 = y0 + pasY;//increment the y position
			}
			//If the x position is after the lcd width
			if (x0 > LCD_WIDTH)
			{
				x0 = LCD_WIDTH;//set the x position to the lcd width
			}
			//If the y position is after the lcd height
			if (y0 > LCD_HEIGHT)
			{
				y0 = LCD_HEIGHT;//set the y position to the lcd height
			}
			//If the x position is before 0
			if (x0 < 0)
			{
				x0 = 0;//set the x position to 0
			}
			//If the y position is before 0
			if (y0 < 0)
			{
				y0 = 0;//set the y position to 0
			}
			
			show_picvar($(curimg), x0, y0, A_XOR);//Shows the cursor
		}
	}
	while(key != 13);//While the user don't press on enter
	
	show_picvar($(curimg), x0, y0, A_XOR);//Erase the cursor
	
	sprintf(coorX, "%d", x0);//Convert x0 to string then copy to coorX
	sprintf(coorY, "%d", y0);//Convert y0 to string then copy to coorY
	
	return (strcat(strcat(coorX,","), coorY));//Copy ',' and coorY after coorX then push the resulting string
}

void BUTTON(int x0, int y0, int x1, int y1, char* titre, int arg)
{	
	PANEL(x0, y0, x1, y1, 1);//Draws a panel to erase the background of the button
	
	BOX(x0, y0, x1, y1, arg);//Draws a box (the sides of the button)
	
	//If arg is not a valid string drawing attribute
	if (arg > 4)
	{
		arg = 4; //Assign the shaded attribute to arg
	}
	
	DRAWSTR((x0+2), (y0+2), titre, arg);//Draws the button caption
}

void LABEL(int x0, int y0, int x1, int y1, char* titre, int arg)
{	
	PANEL(x0, y0, x1, y1, 1);//Draws a panel to erase the background of the label
	
	//If arg is not a valid string drawing attribute
	if (arg > 4)
	{
		arg = 4; //Assign the shaded attribute to arg
	}
	
	DRAWSTR((x0), (y0), titre, arg);//Draws the label caption
}

void WIN(int x0, int y0, int x1, int y1, char* titre, int arg)
{	
	PANEL(x0, y0, x1, y1, 1);//Draws a panel to erase the background of the window
	
	BOX(x0, y0, x1, y1, arg);//Draws a box (the sides of the window)
	
	DRAWLINE((x0+1), (y0+12), (x1-1), (y0+12), arg); //Draws the title bar line
	
	//If arg is not a valid string drawing attribute
	if (arg > 4)
	{
		arg = 4; //Assign the shaded attribute to arg
	}
	
	DRAWSTR((x0+2), (y0+2), titre, arg);//Draws the window caption
}

void window(int x0, int y0, int x1, int y1, char* titre, int arg)
{
	BOX(x0, y0, x1, y1, arg);//Draws a box (the sides of the window)
	
	DRAWLINE((x0+1), (y0+12), (x1-1), (y0+12), arg);//Draws the title bar line
	
	//If arg is not a valid string drawing attribute
	if (arg > 4)
	{
		arg = 4; //Assign the shaded attribute to arg
	}
	
	DRAWSTR((x0+2), (y0+2), titre, arg);//Draws the window caption
}

void REMOVE(char* nomFichier)
{	
	remove(nomFichier);//Deletes the file
}

void REN(char* nomFichier, char* nouveauNom)
{	
	rename(nomFichier, nouveauNom);//Renames the file
}

char* BMPCUR(int x0, int y0, char* fichier, int pasX, int pasY)
{
	char 	coorX[50], //A buffer of 50 bytes to stock the x coordinate (+ ',' and the y coordinate when the function will returns)
				coorY[50], //A buffer of 50 bytes to stock the y coordinate
				*curImg; //The name of the PIC file in TIOS string format
				
	int		key = 0; //Will stock the key typed by the user

	//Converts fichier to TIOS string
	curImg = stringToVar(fichier);

	//Shows the cursor for the first time
	show_picvar(curImg, x0, y0, A_XOR);

	do
	{
		//When the user presses a key
		if (kbhit ( ))
		{
			//Assing the key pressed to key
			key = ngetchx ( );
			//Hides the cursor
			show_picvar(curImg, x0, y0, A_XOR);
			
			//If the key pressed is the right key
			if (key == KEY_RIGHT)
			{
				x0 = x0 + pasX; //increment the x position
			}
			//If the key pressed is the left key
			if (key == KEY_LEFT)
			{
				x0 = x0 - pasX;//decrement the x position
			}
			//If the key pressed is the up key
			if (key == KEY_UP)
			{
				y0 = y0 - pasY;//decrement the y position
			}
			//If the key pressed is the down key
			if (key == KEY_DOWN)
			{
				y0 = y0 + pasY;//increment the y position
			}
			//If the x position is after the lcd width
			if (x0 > LCD_WIDTH)
			{
				x0 = LCD_WIDTH;//set the x position to the lcd width
			}
			//If the y position is after the lcd height
			if (y0 > LCD_HEIGHT)
			{
				y0 = LCD_HEIGHT;//set the y position to the lcd height
			}
			//If the x position is before 0
			if (x0 < 0)
			{
				x0 = 0;//set the x position to 0
			}
			//If the y position is before 0
			if (y0 < 0)
			{
				y0 = 0;//set the y position to 0
			}
			
			//Shows the cursor
			show_picvar(curImg, x0, y0, A_XOR);
		}
	}
	while(key != 13);//While the user don't press on enter
	
	//Erase the cursor
	show_picvar(curImg, x0, y0, A_XOR);
	
	sprintf(coorX, "%d", x0);//Convert x0 to string then copy to coorX
	sprintf(coorY, "%d", y0);//Convert y0 to string then copy to coorY
	
	//Frees the memory allocated to curImg
	free(curImg - strlen(fichier) - 1);
	
	//Copy ',' and coorY after coorX then push the resulting string
	return (strcat(strcat(coorX,","), coorY));
}

void BMPGET(int x0, int y0, int x1, int y1, char* fichier)
{

	BITMAP* bitmap;//A bitmap pointer used to get the desired part of the screen
	SCR_RECT rec;//A rec structure used to give the coordinates of the bitmap
	
	//Assign the coordinates of the bitmap
	rec.xy.x0 = x0;
	rec.xy.y0 = y0;
	rec.xy.x1 = x1;
	rec.xy.y1 = y1;
	
	bitmap = (BITMAP*)malloc(BitmapSize(&rec));//Allocates memory to the bitmap
	BitmapGet (&rec, bitmap);//Gets the bitmap from the screen
	
	savePicvar(fichier, bitmap);//Saves the bitmap in a TIOS PIC variable
	
	free(bitmap);//Frees the memory allocated to the bitmap
}

void BMPPUT(int x0, int y0, char* fichier, int arg)
{
	char  *var;//Will contain the name of the TIOS variable in TIOS string format
	
	var = stringToVar(fichier);//Convert fichier to TIOS string format
	
	show_picvar(var, x0, y0, arg);//Shows the PIC variable
	
	free(var - strlen(fichier) - 1);//Frees the memory allocated to the TIOS string
}

int STRFIND(char* str0, char* str1)
{
	int	ind = -1,//The index of the string in the first string
			pos,//Will be used by the for to browse the first string
			cur,//Will be used to browse a part of the first string
			ptr,//Will be used to mark where begins the string in the first string
			length0,//Will contain the length of the first string
			length1;//Will contain the length of the string to find

	length0 = strlen(str0);//Assign to length0 the length of str0
	length1 = strlen(str1);//Assign to length0 the length of str1

	//for each character of the string, from 0 to the length0 - 1
	for (pos = 0; pos < length0; pos++) {
		cur = pos;//Begin to search from the current position
		ptr = pos;//Mark the initial position
		
		//while cur is lesser than the length of the string to find and the character at cur in the string is equal to the character at cur - ptr in the string to find
		while (str0[cur] == str1[cur - ptr] && cur < length0) {
			cur++;//Increment cur
		}
		
		//If cur-ptr equals the length of the string to find that means the string is found
		if ((cur - ptr) == length1) {
			ind = ptr;//Asign the initial position as index
			break;//Exits the for
		}
	}
	
	return ind;//Returns the index
}

int STRLIND(char* str0, char* str1)
{
	int	ind = -1,//The index of the string in the first string
			pos,//Will be used by the for to browse the first string
			cur,//Will be used to browse a part of the first string
			ptr,//Will be used to mark where begins the string in the first string
			length0,//Will contain the length of the first string
			length1;//Will contain the length of the string to find

	length0 = strlen(str0);//Assign to length0 the length of str0
	length1 = strlen(str1);//Assign to length0 the length of str1

	//for each character of the string, backward, from the length of the string -1 to 0
	for (pos = (length0 - 1); pos >= 0; pos--) {
		cur = pos - length1 + 1;//Begin to search from the current position minus the length of the string to find
		ptr = cur;//Mark the initial position
		
		//while cur is lesser than the length of the string to find and the character at cur in the string is equal to the character at cur - ptr in the string to find
		while (str0[cur] == str1[cur - ptr] && cur < length0) {
			cur++;//Increment cur
		}
		
		//If cur-ptr equals the length of the string to find that means the string is found
		if ((cur - ptr) == length1) {
			ind = ptr;//Asign the initial position as index
			break;//Exits the for
		}
	}
	
	return ind;//Returns the index
}

/*
	Merges 2 string
*/
char* mergestr(char *str0, char *str1, int len0, int len1, int pos)
{
	int i,//Will be used by the for as a counter
	    point0 = 0,//Will contain the current index on the first string
	    point1 = 0;//Will contain the current index on the string to insert
	char *array = malloc(len0 + len1 + 1);//Allocate memory for the new string (+1 is for the null terminating character)
	char array0[len0 + 1];//A buffer to contain the first string
	char array1[len1 + 1];//A buffer to contain the string to insert
	
  strcpy(array0, str0);//Copy the first string in the buffer
  strcpy(array1, str1);//Copy the string to insert in the buffer
  
  //for each characters, from 0 to the sum of the length of both string
  for (i=0; i<(len0 + len1); i++)
  {
  	//If i is lesser than the offset of if i is greater than the length of the string to insert + the offset
   	if ((i < pos && point0 < len0) || (!(i >= pos && point1 < len1) && (point0 < len0)))
   	{
   		array[i] = array0[point0++];//take a char in the first string and insert it in the new string, then increment the current index of the first string
   	}
   	else
   	{
   		array[i] = array1[point1++];//take a char in the string to insert and insert it in the new string, then increment the current index of the string to insert
   	}
  }
  
  array[i] = '\0';

	return array;
}

char* STRINS(char* str0, char* str1, int pos)
{
	char 	 *str;//The new string
	
	str = mergestr(str0, str1, strlen(str0), strlen(str1), pos);//Merges the string
	
	return str;
}

char* STRLOW(char* str)
{
	int i,//Used by the for as a counter
	    len;//Will contain the length of the string
	
	len = strlen(str);//Assign the length of str to len
	
	//For each character of str
	for (i=0; i<len; i++)
	{
		//If the character is uppercase
		if (str[i] <= 90 && str[i] >= 65)
		{
			str[i] = str[i] + 32;//Converts the character to lower case
		}
	}

	return str;
}

char* STRTRIM(char* str)
{
	BOOL fin = FALSE;//A flag that will indicate if there is no more non-space characters
	BOOL trouve;//A flag that will indicate if the first or the last non-space character is found
	int i,//Used by the for as a counter
	    len,//Will contain the length of the string
	    point0,//Will contain the index of the first non-space character
	    point1;//Will contain the index of the last non-space character
	
	len = strlen(str);//Assign the length of str to len
	
	point0 = 0;//Initialize the forward index
	point1 = len - 1;//Initialize the backward index
	
	/*
		Removes all the spaces at the begining
	*/
	trouve = FALSE;
	
	for (i=0; i<len && !trouve; i++)
	{
		if (str[i] != ' ')
		{
			point0 = i;
			trouve = TRUE;
		}
	}
	
	/*
		Removes all the spaces at the begining
	*/	
	trouve = FALSE;
	
	for (i=len - 1; i>=0 && !trouve; i--)
	{
		if (str[i] != ' ')
		{
			point1 = i;
			trouve = TRUE;
		}
	}
	
	//for each character of the trimmed string, from 0 to the size of the non-space characters
	for (i=0; i<len && !fin; i++)
	{
		//If the character isn't a space
		if (i >= point0 && i <= point1)
		{
			str[i - point0] = str[i];//Adds the character to the string
		}
		//Else if the character index is greater than the last index of non-space characters
		else if (i > point1)
		{
			fin = TRUE;//Exits the for next loop
		}
	}
	
	str[point1 - point0 + 1] = '\0';//Adds the null terminating character
	
	return str;
}

char* SUBSTR(char* str, int offset, int length)
{
	char 	 *sub;//The substring to be returned
	int i,//A simple counter that will be used by the for
	    len;//Will contain the length of the string
	
	len = strlen(str);//Assign the length of the string to len
	
	//If the parameters are OK ...
	if (length <= (len - offset) && length > 0) {
		sub = malloc(length + 1); //Assign length + 1 bytes of memory to sub (+1 is for the null terminating character)
		//For each character in the string, from the offset to the offset + length of the substring -1
		for (i = offset; i < offset + length; i++) {
			sub[i - offset] = str[i];//Assing the character at position i in the string to the position i - offset in the substring
		}
		sub[i - offset] = '\0';//Ends the substring by the null character
	}
	else {
		sub = NULL;
	}
	
	return sub;
}
