/*
		Copyright 2000-2002 Frdric Brown
*/

/*
		This file is part of CLib.

    CLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with CLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Available Patches
// #define USE_GRAY_PATCH
// #define USE_LONGMUL_PATCH
// #define USE_LONGDIV_PATCH
// #define USE_INITS_PATCH

#define RETURN_VALUE // Return Pushed Expression

#include <nostub.h>  // Make NoStub Program
#include <all.h>     // Include All Header Files

int _ti89;           // Produce .89Z File
int _ti92plus;       // Produce .9XZ File

/*
	Converts an ANSI string to the TIOS string format
*/
char* stringToVar(char* string)
{
	int length = strlen(string);//The length of the string to convert
	char *var = malloc(sizeof(char) * (length + 2));//Allocate length bytes to var, the pointer to the TIOS formated string
	int i;//A counter that will be used by the for
	
	var[0] = '\0';//TIOS strings begin with null character
	
	//for each character position of var, from 1 up to the length of the string to convert
	for (i=1; i<=length; i++) {
		var[i] = string[i-1];//Assign to the character position i of var the character in position i-1 of the string to convert
	}
	
	var[i] = '\0';//TIOS strings end with null character
	
	return (var + length + 1);//Return a char pointer to the ending null character of the TIOS string
}

/*
 Put a bitmap in the screen from a PIC variable
*/
void show_picvar (char *SymName, int x, int y, int Attr) {
	SYM_ENTRY *sym_entry = SymFindPtr (SymName, 0);//The pointer on the TIOS variable
	//If the TIOS variable doesn't exists
	if (!sym_entry) return;//Quits the function
	//If the TIOS variable isn't a PIC
	if (peek (HToESI (sym_entry->handle)) != PIC_TAG) return;//Quits the function
	BitmapPut (x, y, HeapDeref (sym_entry->handle) + 2, ScrRect, Attr);//Shows the bitmap
}

// Main Function
void _main()
{
	char 	coorX[50], //A buffer of 50 bytes to stock the x coordinate (+ ',' and the y coordinate when the function will returns)
				coorY[50], //A buffer of 50 bytes to stock the y coordinate
				*fichier, //Argument 3: the name of the PIC file
				*curImg; //The name of the PIC file in TIOS string format
				
	int		x0, //Argument 1: the upperleft x coordinate
				y0, //Argument 2: the upperleft y coordinate
				key = 0, //Will stock the key typed by the user
				pasX, //Argument 4: the horizontal speed
				pasY; //Argument 5: the vertical speed
	
	ESI argptr;//A pointer on the argument stack
	InitArgPtr(argptr);//Initialization of the argument pointer
	
	x0 = GetIntArg(argptr); //Assign argument 1 to x0 as int
	y0 = GetIntArg(argptr); //Assign argument 2 to y0 as int
	fichier = GetStrnArg(argptr); //Assign argument 3 to fichier as string
	pasX = GetIntArg(argptr); //Assign argument 4 to pasX as int
	pasY = GetIntArg(argptr); //Assign argument 5 to pasY as int

	//Converts fichier to TIOS string
	curImg = stringToVar(fichier);

	//Shows the cursor for the first time
	show_picvar(curImg, x0, y0, A_XOR);

	do
	{
		//When the user presses a key
		if (kbhit ( ))
		{
			//Assing the key pressed to key
			key = ngetchx ( );
			//Hides the cursor
			show_picvar(curImg, x0, y0, A_XOR);
			
			//Depending on the key pressed by the user (can't use a switch here, because KEY_... are macros instead of constants)
			//If the key pressed is the right key
			if (key == KEY_RIGHT)
			{
				x0 = x0 + pasX; //increment the x position
			}
			//If the key pressed is the left key
			else if (key == KEY_LEFT)
			{
				x0 = x0 - pasX;//decrement the x position
			}
			//If the key pressed is the up key
			else if (key == KEY_UP)
			{
				y0 = y0 - pasY;//decrement the y position
			}
			//If the key pressed is the down key
			else if (key == KEY_DOWN)
			{
				y0 = y0 + pasY;//increment the y position
			}
			//If the x position is after the lcd width
			if (x0 > LCD_WIDTH)
			{
				x0 = LCD_WIDTH;//set the x position to the lcd width
			}
			//If the y position is after the lcd height
			if (y0 > LCD_HEIGHT)
			{
				y0 = LCD_HEIGHT;//set the y position to the lcd height
			}
			//If the x position is before 0
			if (x0 < 0)
			{
				x0 = 0;//set the x position to 0
			}
			//If the y position is before 0
			if (y0 < 0)
			{
				y0 = 0;//set the y position to 0
			}
			
			//Shows the cursor
			show_picvar(curImg, x0, y0, A_XOR);
		}
	}
	while(key != 13);//While the user don't press on enter
	
	//Erase the cursor
	show_picvar(curImg, x0, y0, A_XOR);
	
	sprintf(coorX, "%d", x0);//Convert x0 to string then copy to coorX
	sprintf(coorY, "%d", y0);//Convert y0 to string then copy to coorY
	
	//Copy ',' and coorY after coorX then push the resulting string
	push_ANSI_string(strcat(strcat(coorX,","), coorY));
	
	//Frees the memory allocated to curImg
	free(curImg - strlen(fichier) - 1);
}
