/*
		Copyright 2000-2002 Frdric Brown
*/

/*
		This file is part of CLib.

    CLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with CLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

// #define SAVE_SCREEN        // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

short _ti89;                  // Produce .89Z File
short _ti92plus;              // Produce .9XZ File

/*
	Converts an ANSI string to the TIOS string format
*/
char* stringToVar(char* string)
{
	int length = strlen(string);//The length of the string to convert
	char *var = malloc(sizeof(char) * (length + 2));//Allocate length bytes to var, the pointer to the TIOS formated string
	int i;//A counter that will be used by the for
	
	var[0] = '\0';//TIOS strings begin with null character
	
	//for each character position of var, from 1 up to the length of the string to convert
	for (i=1; i<=length; i++) {
		var[i] = string[i-1];//Assign to the character position i of var the character in position i-1 of the string to convert
	}
	
	var[i] = '\0';//TIOS strings end with null character
	
	return (var + length + 1);//Return a char pointer to the ending null character of the TIOS string
}

/*
	Saves a bitmap to a TIOS PIC variable
*/
void savePicvar (char *nom, BITMAP* bmp) {
	short taille = bmp->NumRows*((bmp->NumCols+7)>>3)+4;//The length of the bitmap
	SYM_ENTRY *sym_entry;//A pointer on the TIOS VAT entry
	char *vat,//A pointer to the TIOS variable
			 *var;//Will contain the name of the variable, in TIOS string format

	var = stringToVar(nom);//Convert nom to TIOS string format
	
	sym_entry = DerefSym(SymAdd(var)) ;//Creates a new TIOS entry and assigns it to sym_entry
	
	vat = (unsigned char*)HeapDeref(sym_entry->handle = HeapAlloc(taille+3)) ;//Assigns the adress of the TIOS variable to vat
	//Writes the dimensions of the PIC to the PIC header
	vat[0] =  (taille + 1) / 256;
	vat[1] =  (taille + 1) % 256;
	memcpy(vat+2, bmp, taille) ;//Copy the bitmap data to the TIOS variable
	vat[taille+2] = 0xDF ;//Ends the TIOS variable by the PIC_VAR tag (DF)
	
	free(var - strlen(nom) - 1);//Frees the memory allocated to the TIOS string
}

// Main Function
void _main(void)
{
	int x0,//Argument 1: the upperleft x coordinate
			y0,//Argument 2: the upperleft y coordinate
			x1,//Argument 3: the downright x coordinate
			y1;//Argument 4: the downright y coordinate
	char *fichier;//Argument 5: the TIOS variable name

	BITMAP* bitmap;//A bitmap pointer used to get the desired part of the screen
	SCR_RECT rec;//A rec structure used to give the coordinates of the bitmap
	
	ESI argptr;//A pointer on the argument stack
	InitArgPtr(argptr);//Initialization of the argument pointer
	x0 = GetIntArg(argptr);//Assign argument 1 to x0 as int
	y0 = GetIntArg(argptr);//Assign argument 2 to y0 as int
	x1 = GetIntArg(argptr);//Assign argument 3 to x1 as int
	y1 = GetIntArg(argptr);//Assign argument 4 to y1 as int
	fichier = GetStrnArg(argptr);//Assign argument 5 to fichier as string
	
	//Assign the coordinates of the bitmap
	rec.xy.x0 = x0;
	rec.xy.y0 = y0;
	rec.xy.x1 = x1;
	rec.xy.y1 = y1;
	
	bitmap = (BITMAP*)malloc(BitmapSize(&rec));//Allocates memory to the bitmap
	BitmapGet (&rec, bitmap);//Gets the bitmap from the screen
	
	savePicvar(fichier, bitmap);//Saves the bitmap in a TIOS PIC variable
	
	free(bitmap);//Frees the memory allocated to the bitmap
}
