/*
		Copyright 2000-2002 Frdric Brown
*/

/*
		This file is part of CLib.

    CLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with CLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

// #define SAVE_SCREEN        // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

short _ti89;                  // Produce .89Z File
short _ti92plus;              // Produce .9XZ File

/*
	Converts an ANSI string to the TIOS string format
*/
char* stringToVar(char* string)
{
	int length = strlen(string);//The length of the string to convert
	char *var = malloc(sizeof(char) * (length + 2));//Allocate length bytes to var, the pointer to the TIOS formated string
	int i;//A counter that will be used by the for
	
	var[0] = '\0';//TIOS strings begin with null character
	
	//for each character position of var, from 1 up to the length of the string to convert
	for (i=1; i<=length; i++) {
		var[i] = string[i-1];//Assign to the character position i of var the character in position i-1 of the string to convert
	}
	
	var[i] = '\0';//TIOS strings end with null character
	
	return (var + length + 1);//Return a char pointer to the ending null character of the TIOS string
}

/*
 Put a bitmap in the screen from a PIC variable
*/
void show_picvar (char *SymName, int x, int y, int Attr) {
	SYM_ENTRY *sym_entry = SymFindPtr (SymName, 0);//The pointer on the TIOS variable
	//If the TIOS variable doesn't exists
	if (!sym_entry) return;//Quits the function
	//If the TIOS variable isn't a PIC
	if (peek (HToESI (sym_entry->handle)) != PIC_TAG) return;//Quits the function
	BitmapPut (x, y, HeapDeref (sym_entry->handle) + 2, ScrRect, Attr);//Shows the bitmap
}

// Main Function
void _main(void)
{
	int x0,//Argument 1: the upperleft x coordinate
			y0;//Argument 2: the upperleft y coordinate
	char *fichier,//Argument 3: the TIOS variable name
			 *var;//Will contain the name of the TIOS variable in TIOS string format
	int	arg;//Argument 4: The attribute for painting
	
	
	ESI argptr;//A bitmap pointer used to get the desired part of the screen
	InitArgPtr(argptr);////Initialization of the argument pointer
	x0 = GetIntArg(argptr);//Assign argument 1 to x0 as int
	y0 = GetIntArg(argptr);//Assign argument 2 to y0 as int
	fichier = GetStrnArg(argptr);//Assign argument 3 to fichier as string
	arg = GetIntArg(argptr);//Assign argument 4 to arg as int
	
	var = stringToVar(fichier);//Convert fichier to TIOS string format
	
	show_picvar(var, x0, y0, arg);//Shows the PIC variable
	
	free(var - strlen(fichier) - 1);//Frees the memory allocated to the TIOS string
}
