/*
		Copyright 2000-2002 Frdric Brown
*/

/*
		This file is part of CLib.

    CLib is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with CLib; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

// #define SAVE_SCREEN        // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files
void DRAWLINE(int x0, int y0, int x1, int y1, int arg)
{
	//Depending on arg
	switch (arg) {
		case 0: {
			DrawLine(x0, y0, x1, y1, A_NORMAL);//Draws a normal line
			break;
		}
		case 1: {
			DrawLine(x0, y0, x1, y1, A_REVERSE);//Draws a reverse line
			break;
		}
		case 2: {
			DrawLine(x0, y0, x1, y1, A_XOR);//Draws a xor line
			break;
		}
		case 3: {
			DrawLine(x0, y0, x1, y1, A_THICK1);//Draws a double thick line
			break;
		}
		case 4: {
			DrawLine(x0, y0, x1, y1, A_SHADE_V);//Draws the line using a vertical shading pattern
			break;
		}
		case 5: {
			DrawLine(x0, y0, x1, y1, A_SHADE_H);//Draws the line using a horizontal shading pattern
			break;
		}
		case 6: {
			DrawLine(x0, y0, x1, y1, A_SHADE_NS);//Draws the line using a negative slope diagonal shading pattern
			break;
		}
		case 7: {
			DrawLine(x0, y0, x1, y1, A_SHADE_PS);//Draws the line using a positive slope diagonal shading pattern
			break;
		}
	}
}

void BOX(int x0, int y0, int x1, int y1, int arg)
{
	DRAWLINE(x0, y0, x1, y0, arg);//Draws the top side
	DRAWLINE(x0, y1, x1, y1, arg);//Draws the bottom side
	DRAWLINE(x0, (y0+1), x0, (y1-1), arg);//Draws the left side
	DRAWLINE(x1, (y0+1), x1, (y1-1), arg);//Draws the right side
}

void DRAWSTR(int x, int y, char* chaine, int arg)
{	
	//Depending on arg
	switch(arg) {
		case 0: {
			DrawStr(x, y, chaine, A_NORMAL);//Draws a normal string
			break;
		}
		case 1: {
			DrawStr(x, y, chaine, A_REVERSE);//Draws a opaque white on black string
			break;
		}
		case 2: {
			DrawStr(x, y, chaine, A_REPLACE);//Draws a opaque black on white string
			break;
		}
		case 3: {
			DrawStr(x, y, chaine, A_XOR);//Draws a xor string
			break;
		}
		case 4: {
			DrawStr(x, y, chaine, A_SHADED);//Draws a shaded string
			break;
		}
	}
}

void PANEL(int x0, int y0, int x1, int y1, int arg)
{
	int i;//Will be used by the for as counter
	
	//For each pixels, from y0 to y1
	for (i=y0; i<=y1; i++)
	{
		DRAWLINE(x0, i, x1, i, arg);//Draws a horizontal line from (x0,i) to (x1,i)
	}
}

short _ti89;                  // Produce .89Z File
short _ti92plus;              // Produce .9XZ File

// Main Function
void _main(void)
{
	int		x0,//Argument 1: the upperleft x coordinate
			y0,//Argument 2: the upperleft y coordinate
			x1,//Argument 3: the downright x coordinate
			y1,//Argument 4: the downright y coordinate
			arg;//Argument 6: the drawing patern
	char	*titre;//Argument 5: the caption
	
	ESI argptr;//A pointer on the argument stack
	InitArgPtr(argptr);//Initialization of the argument pointer
	
	x0 = GetIntArg(argptr);//Assign argument 1 to x0 as int
	y0 = GetIntArg(argptr);//Assign argument 2 to y0 as int
	x1 = GetIntArg(argptr);//Assign argument 3 to x1 as int
	y1 = GetIntArg(argptr);//Assign argument 4 to y1 as int
	titre = GetStrnArg(argptr);//Assign argument 5 to titre as string
	arg = GetIntArg(argptr);//Assign argument 6 to arg as int
	
	BOX(x0, y0, x1, y1, arg);//Draws a box (the sides of the window)
	
	DRAWLINE((x0+1), (y0+12), (x1-1), (y0+12), arg);//Draws the title bar line
	
	//If arg is not a valid string drawing attribute
	if (arg > 4)
	{
		arg = 4; //Assign the shaded attribute to arg
	}
	
	DRAWSTR((x0+2), (y0+2), titre, arg);//Draws the window caption
}
